import type { ArgConfig, ArgsParserInterface, ParsedArgumentsCollectionInterface } from "./types";
/**
 * A collection of parsed arguments.
 *
 * @category Classes
 */
export declare class ParsedArgumentsCollection implements ParsedArgumentsCollectionInterface {
    /**
     * A collection of parsed positional arguments.
     */
    private readonly positionalArgs;
    /**
     * A collection of parsed optional arguments.
     */
    private readonly optionalArgs;
    /**
     * Returns all arguments as a record.
     */
    get positional(): Record<string, unknown>;
    /**
     * Returns all arguments as a record.
     */
    get optional(): Record<string, unknown>;
    /**
     * Adds a new argument to the collection.
     * @param name - The name of the argument.
     * @param value - The value of the argument.
     * @returns The instance of ParsedArgumentsCollection for chaining.
     */
    add(name: string, value: unknown): this;
    /**
     * Retrieves an argument value.
     * @param name - The name of the argument.
     * @returns The value of the argument, or undefined if not found.
     */
    get<T = unknown>(name: string): T;
    /**
     * Checks if an argument exists in the collection.
     * @param name - The name of the argument.
     * @returns True if the argument exists, otherwise false.
     */
    has(name: string): boolean;
    /**
     * Formats the argument name by removing leading dashes.
     * @param name - The argument name.
     * @returns The formatted name.
     */
    private formatName;
    /**
     * Checks if the argument is positional.
     * @param name - The argument name.
     * @returns True if the argument is positional, otherwise false.
     */
    private isPositional;
}
/**
 * Parser for command-line arguments.
 *
 * @category Classes
 */
export declare class ArgsParser implements ArgsParserInterface {
    /**
     * Maps argument names to their configuration.
     */
    private readonly argsMap;
    /**
     * Maps argument aliases to their corresponding names.
     */
    private readonly aliasMap;
    /**
     * Tracks used argument names and aliases to prevent duplicates.
     */
    private readonly usedArgs;
    /**
     * Constructs an instance of ArgsParser with the provided argument configurations.
     * @param args - An array of argument configurations.
     */
    constructor(args?: ArgConfig[]);
    /**
     * Retrieves the help message.
     */
    get help(): string;
    /**
     * Adds the help message for the arguments to the parser.
     * @returns The updated parser.
     */
    addHelp(): ArgsParser;
    /**
     * Adds a new argument configuration to the parser.
     * @param config - The argument configuration.
     * @returns The instance of ArgsParser for chaining.
     */
    addArgument(config: ArgConfig): ArgsParser;
    /**
     * Parses the given argument string and returns a collection of parsed arguments.
     * @param argv - The argument string.
     * @returns A ParsedArgumentsCollection containing the parsed arguments.
     */
    parse(argv: string[]): ParsedArgumentsCollection;
    /**
     * Checks the validity of an argument configuration.
     * @param config - The argument configuration.
     * @throws AddArgumentError if the configuration is invalid.
     */
    private checkArgumentConfig;
    /**
     * Checks the number of positional arguments.
     * @param values - An array of positional argument values.
     * @param argName - The name of the argument.
     * @param nargsConfig - The nargs configuration.
     */
    private checkEnoughPositionalValues;
    private getArgsCountToRead;
    /**
     * Retrieves the argument configuration for a given key.
     * @param key - The argument key.
     * @returns The corresponding ArgConfig.
     * @throws ArgumentNameError if the argument key is unknown.
     */
    private getArgConfig;
    /**
     * Retrieves the positional arguments.
     * @returns An array of positional argument configurations.
     */
    private getPositionalArguments;
    /**
     * Retrieves the optional arguments.
     * @returns An array of optional argument configurations.
     */
    private getOptionalArguments;
    /**
     * Checks if an argument is optional.
     * @param config - The argument configuration.
     * @returns True if the argument is optional, otherwise false.
     */
    private isOptional;
}
