import { AddArgumentError, ArgumentNameError, ArgumentValueError } from "./exceptions";
import { buildNArgsConfig, castArgValue, parseArgsArray, validateCastedArgValue } from "./utils/utils";
import { convertToTable, formatArgHelp, tabTableRows } from "./utils/help";
/**
 * A collection of parsed arguments.
 *
 * @category Classes
 */
export class ParsedArgumentsCollection {
    constructor() {
        /**
         * A collection of parsed positional arguments.
         */
        this.positionalArgs = {};
        /**
         * A collection of parsed optional arguments.
         */
        this.optionalArgs = {};
    }
    /**
     * Returns all arguments as a record.
     */
    get positional() {
        return Object.assign({}, this.positionalArgs);
    }
    /**
     * Returns all arguments as a record.
     */
    get optional() {
        return Object.assign({}, this.optionalArgs);
    }
    /**
     * Adds a new argument to the collection.
     * @param name - The name of the argument.
     * @param value - The value of the argument.
     * @returns The instance of ParsedArgumentsCollection for chaining.
     */
    add(name, value) {
        const formattedName = this.formatName(name);
        if (this.isPositional(name)) {
            this.positionalArgs[formattedName] = value;
        }
        else {
            this.optionalArgs[formattedName] = value;
        }
        return this;
    }
    /**
     * Retrieves an argument value.
     * @param name - The name of the argument.
     * @returns The value of the argument, or undefined if not found.
     */
    get(name) {
        var _a, _b;
        const formattedName = this.formatName(name);
        if (this.isPositional(name)) {
            return ((_a = this.positional[formattedName]) !== null && _a !== void 0 ? _a : undefined);
        }
        return ((_b = this.optionalArgs[formattedName]) !== null && _b !== void 0 ? _b : undefined);
    }
    /**
     * Checks if an argument exists in the collection.
     * @param name - The name of the argument.
     * @returns True if the argument exists, otherwise false.
     */
    has(name) {
        var _a, _b;
        const formattedName = this.formatName(name);
        if (this.isPositional(name)) {
            return ((_a = this.positional[formattedName]) !== null && _a !== void 0 ? _a : undefined) !== undefined;
        }
        return ((_b = this.optionalArgs[formattedName]) !== null && _b !== void 0 ? _b : undefined) !== undefined;
    }
    /**
     * Formats the argument name by removing leading dashes.
     * @param name - The argument name.
     * @returns The formatted name.
     */
    formatName(name) {
        return name.replace(/^--/, '');
    }
    /**
     * Checks if the argument is positional.
     * @param name - The argument name.
     * @returns True if the argument is positional, otherwise false.
     */
    isPositional(name) {
        return !name.startsWith('--');
    }
}
/**
 * Parser for command-line arguments.
 *
 * @category Classes
 */
export class ArgsParser {
    /**
     * Constructs an instance of ArgsParser with the provided argument configurations.
     * @param args - An array of argument configurations.
     */
    constructor(args = []) {
        /**
         * Maps argument names to their configuration.
         */
        this.argsMap = new Map();
        /**
         * Maps argument aliases to their corresponding names.
         */
        this.aliasMap = new Map();
        /**
         * Tracks used argument names and aliases to prevent duplicates.
         */
        this.usedArgs = new Set();
        for (const arg of args) {
            this.addArgument(arg);
        }
    }
    /**
     * Retrieves the help message.
     */
    get help() {
        const positionalArgs = this.getPositionalArguments();
        const optionalArgs = this.getOptionalArguments();
        const positionalArgsRows = [];
        for (const arg of positionalArgs) {
            const nargsConfig = buildNArgsConfig(arg);
            positionalArgsRows.push(...formatArgHelp(arg, nargsConfig));
        }
        const optionalArgsRows = [];
        for (const arg of optionalArgs) {
            const nargsConfig = buildNArgsConfig(arg);
            optionalArgsRows.push(...formatArgHelp(arg, nargsConfig));
        }
        const result = [];
        if (positionalArgsRows.length > 0) {
            result.push(['Positional arguments:'], [''], ...tabTableRows(positionalArgsRows));
        }
        if (optionalArgsRows.length > 0) {
            result.push(['Optional arguments:'], [''], ...tabTableRows(optionalArgsRows));
        }
        return convertToTable(result, 1);
    }
    /**
     * Adds the help message for the arguments to the parser.
     * @returns The updated parser.
     */
    addHelp() {
        return this.addArgument({
            name: '--help',
            alias: '-h',
            description: 'Show help',
            type: 'boolean',
            const: true,
            default: false,
        });
    }
    /**
     * Adds a new argument configuration to the parser.
     * @param config - The argument configuration.
     * @returns The instance of ArgsParser for chaining.
     */
    addArgument(config) {
        this.checkArgumentConfig(config);
        this.argsMap.set(config.name, config);
        this.usedArgs.add(config.name);
        if (config.alias !== undefined) {
            this.aliasMap.set(config.alias, config.name);
            this.usedArgs.add(config.alias);
        }
        return this;
    }
    /**
     * Parses the given argument string and returns a collection of parsed arguments.
     * @param argv - The argument string.
     * @returns A ParsedArgumentsCollection containing the parsed arguments.
     */
    parse(argv) {
        const positionalArgs = this.getPositionalArguments();
        const optionalArgs = new Set(this.getOptionalArguments());
        let [parsedPositional, parsedOptional] = parseArgsArray(argv);
        parsedPositional.reverse();
        const result = new ParsedArgumentsCollection();
        for (const argConfig of positionalArgs) {
            const nargsConfig = buildNArgsConfig(argConfig);
            this.checkEnoughPositionalValues(parsedPositional, argConfig.name, nargsConfig);
            const toReadCount = this.getArgsCountToRead(nargsConfig, parsedPositional.length);
            const value = [];
            for (let i = 0; i < toReadCount; ++i) {
                if (parsedPositional.length === 0) {
                    break;
                }
                value.push(parsedPositional.pop());
            }
            const castedValue = castArgValue(value, argConfig, nargsConfig, value.length > 0);
            result.add(argConfig.name, validateCastedArgValue(castedValue, argConfig, nargsConfig));
        }
        if (parsedPositional.length > 0) {
            throw new ArgumentValueError('Too many positional arguments.');
        }
        for (const [key, value] of Object.entries(parsedOptional)) {
            const argConfig = this.getArgConfig(key);
            const nargsConfig = buildNArgsConfig(argConfig);
            const castedValue = castArgValue(value, argConfig, nargsConfig);
            result.add(argConfig.name, validateCastedArgValue(castedValue, argConfig, nargsConfig));
            optionalArgs.delete(argConfig);
        }
        for (const argConfig of optionalArgs) {
            const nargsConfig = buildNArgsConfig(argConfig);
            const castedValue = castArgValue([], argConfig, nargsConfig, false);
            result.add(argConfig.name, validateCastedArgValue(castedValue, argConfig, nargsConfig));
        }
        return result;
    }
    /**
     * Checks the validity of an argument configuration.
     * @param config - The argument configuration.
     * @throws AddArgumentError if the configuration is invalid.
     */
    checkArgumentConfig(config) {
        if (config.name.startsWith('-') && !config.name.startsWith('--')) {
            throw new AddArgumentError(`Argument name ${config.name} is invalid.`);
        }
        if (config.alias !== undefined && (!config.alias.startsWith('-') || config.alias.startsWith('--'))) {
            throw new AddArgumentError(`Argument name ${config.name} is invalid.`);
        }
        if (this.usedArgs.has(config.name)) {
            throw new AddArgumentError(`Argument ${config.name} already exists.`);
        }
        if (config.alias !== undefined && this.usedArgs.has(config.alias)) {
            throw new AddArgumentError(`Argument ${config.alias} already exists.`);
        }
    }
    /**
     * Checks the number of positional arguments.
     * @param values - An array of positional argument values.
     * @param argName - The name of the argument.
     * @param nargsConfig - The nargs configuration.
     */
    checkEnoughPositionalValues(values, argName, nargsConfig) {
        const errorMessage = `Not enough positional arguments. ${argName} is required.`;
        if (!nargsConfig.multiple) {
            if (!nargsConfig.allowEmpty && values.length === 0) {
                throw new ArgumentValueError(errorMessage);
            }
            return;
        }
        if (!nargsConfig.allowEmpty && values.length === 0) {
            throw new ArgumentValueError(errorMessage);
        }
        if (!nargsConfig.allowEmpty && nargsConfig.count !== undefined && nargsConfig.count > values.length) {
            throw new ArgumentValueError(errorMessage);
        }
    }
    getArgsCountToRead(nargsConfig, totalCount) {
        if (!nargsConfig.multiple) {
            return 1;
        }
        if (nargsConfig.count !== undefined) {
            return nargsConfig.count;
        }
        return totalCount;
    }
    /**
     * Retrieves the argument configuration for a given key.
     * @param key - The argument key.
     * @returns The corresponding ArgConfig.
     * @throws ArgumentNameError if the argument key is unknown.
     */
    getArgConfig(key) {
        var _a;
        const config = this.argsMap.get((_a = this.aliasMap.get(key)) !== null && _a !== void 0 ? _a : key);
        if (config === undefined) {
            throw new ArgumentNameError(`Unknown argument ${key}.`);
        }
        return config;
    }
    /**
     * Retrieves the positional arguments.
     * @returns An array of positional argument configurations.
     */
    getPositionalArguments() {
        return [...this.argsMap.values()].filter((x) => !this.isOptional(x));
    }
    /**
     * Retrieves the optional arguments.
     * @returns An array of optional argument configurations.
     */
    getOptionalArguments() {
        return [...this.argsMap.values()].filter((x) => this.isOptional(x));
    }
    /**
     * Checks if an argument is optional.
     * @param config - The argument configuration.
     * @returns True if the argument is optional, otherwise false.
     */
    isOptional(config) {
        return config.name.startsWith('--');
    }
}
//# sourceMappingURL=classes.js.map